
import fcntl
import os
from datetime import datetime, timedelta
from threading import TIMEOUT_MAX

import records
import six
from apscheduler.events import (EVENT_JOB_MAX_INSTANCES, EVENT_JOB_SUBMITTED,
                                JobSubmissionEvent)
from apscheduler.executors.base import MaxInstancesReachedError
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.schedulers.base import STATE_PAUSED
from apscheduler.util import timedelta_seconds
from notejob.center.configs import configs


class CuBackgroundScheduler(BackgroundScheduler):

    def _dbs(self):
        url = configs('cron_job_log_db_url')
        db = records.Database(url)
        db = db.get_connection()
        return db

    def update_cron_info(self, cron_id):
        try:
            cron_id = cron_id.split('_')[-1]
            self._dbs().query("update cron_infos set status=-1 where id='%s'" % cron_id)
        except:
            pass

    def _process_jobs(self):
        """
        Iterates through jobs in every jobstore, starts jobs that are due and figures out how long
        to wait for the next round.

        If the ``get_due_jobs()`` call raises an exception, a new wakeup is scheduled in at least
        ``jobstore_retry_interval`` seconds.

        """
        f = open("scheduler.lock", "wb")
        wait_seconds = None
        try:
            fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
        except Exception as exc:
            f.close()
        else:
            if self.state == STATE_PAUSED:
                self._logger.debug('Scheduler is paused -- not processing jobs')
                return None

            self._logger.debug('Looking for jobs to run and os.pid is {%s}' % os.getpid())
            now = datetime.now(self.timezone)
            next_wakeup_time = None
            events = []

            with self._jobstores_lock:
                for jobstore_alias, jobstore in six.iteritems(self._jobstores):
                    try:
                        due_jobs = jobstore.get_due_jobs(now)
                        self._logger.info("due_jobs:%s     os.pid: %s\n" % (len(due_jobs), os.getpid()))
                    except Exception as e:
                        # Schedule a wakeup at least in jobstore_retry_interval seconds
                        self._logger.warning('Error getting due jobs from job store %r: %s',
                                             jobstore_alias, e)
                        retry_wakeup_time = now + timedelta(seconds=self.jobstore_retry_interval)
                        if not next_wakeup_time or next_wakeup_time > retry_wakeup_time:
                            next_wakeup_time = retry_wakeup_time

                        continue

                    for job in due_jobs:
                        # Look up the job's executor
                        try:
                            executor = self._lookup_executor(job.executor)
                        except BaseException:
                            self._logger.error(
                                'Executor lookup ("%s") failed for job "%s" -- removing it from the '
                                'job store', job.executor, job)
                            self.update_cron_info(job.id)
                            self.remove_job(job.id, jobstore_alias)
                            continue

                        run_times = job._get_run_times(now)
                        run_times = run_times[-1:] if run_times and job.coalesce else run_times
                        if run_times:
                            try:
                                executor.submit_job(job, run_times)
                            except MaxInstancesReachedError:
                                self._logger.warning(
                                    'Execution of job "%s" skipped: maximum number of running '
                                    'instances reached (%d)', job, job.max_instances)
                                event = JobSubmissionEvent(EVENT_JOB_MAX_INSTANCES, job.id,
                                                           jobstore_alias, run_times)
                                events.append(event)
                            except BaseException:
                                self._logger.exception('Error submitting job "%s" to executor "%s"',
                                                       job, job.executor)
                            else:
                                event = JobSubmissionEvent(EVENT_JOB_SUBMITTED, job.id, jobstore_alias,
                                                           run_times)
                                events.append(event)

                            # Update the job if it has a next execution time.
                            # Otherwise remove it from the job store.
                            job_next_run = job.trigger.get_next_fire_time(run_times[-1], now)
                            if job_next_run:
                                job._modify(next_run_time=job_next_run)
                                jobstore.update_job(job)
                            else:
                                try:
                                    self.update_cron_info(job.id)
                                    self.remove_job(job.id, jobstore_alias)
                                except:
                                    self._logger.error('Error remove job "%s" to executor "%s"',
                                                       job, job.executor)

                    # Set a new next wakeup time if there isn't one yet or
                    # the jobstore has an even earlier one
                    jobstore_next_run_time = jobstore.get_next_run_time()
                    if jobstore_next_run_time and (next_wakeup_time is None or
                                                   jobstore_next_run_time < next_wakeup_time):
                        next_wakeup_time = jobstore_next_run_time.astimezone(self.timezone)

            # Dispatch collected events
            for event in events:
                self._dispatch_event(event)

            # Determine the delay until this method should be called again
            if self.state == STATE_PAUSED:
                self._logger.debug('Scheduler is paused; waiting until resume() is called')
            elif next_wakeup_time is None:
                self._logger.debug('No jobs; waiting until a job is added')
            else:
                wait_seconds = min(max(timedelta_seconds(next_wakeup_time - now), 0), TIMEOUT_MAX)
                self._logger.debug('Next wakeup is due at %s (in %f seconds)', next_wakeup_time,
                                   wait_seconds)

            fcntl.flock(f, fcntl.LOCK_UN)
            f.close()

        return wait_seconds




from notejob.center.app.core import db


class JobLog(db.Model):
    __tablename__ = 'job_log'
    id = db.Column(db.Integer, primary_key=True)
    log_id = db.Column(db.String(65), nullable=False, index=True,
                       server_default='', default='log id 用uuid生成唯一id,用来用户更新')
    cron_info_id = db.Column(db.Integer, nullable=False, default=0, index=True)
    content = db.Column(db.TEXT, nullable=False, default='', doc='返回的内容')
    create_time = db.Column(db.String(25), nullable=False, default='')
    take_time = db.Column(db.String(25), default='', doc='耗时时间')

    def to_json(self):
        return {
            'id': self.id,
            'job_id': self.job_id,
            'remark': self.remark,
            'content': self.content,
            'traces': self.traces,
            'status': self.status,
            'create_time': self.create_time
        }




import datetime
import time

'''
获取当前时间
'''


def get_now_time(format='%Y-%m-%d %H:%M:%S'):
    return time.strftime(format, time.localtime(time.time()))


'''
获取几秒后的时间 seconds days minutes
'''


def get_next_time(format='%Y-%m-%d %H:%M:%S', **ke):
    tiimes = (datetime.datetime.now() + datetime.timedelta(**ke)).strftime(format)
    return tiimes


def get_today(format='%Y-%m-%d'):
    return time.strftime(format, time.localtime(time.time()))



import json
import time
import traceback
import uuid

import records
import requests
from flask import current_app
from notejob.center.app.common.functions import single_task, wechat_info_err
from notejob.center.app.core import db, scheduler
from notejob.center.configs import configs
from notejob.center.datas.model.cron_infos import CronInfos
from notejob.center.datas.model.job_log import JobLog
from notejob.center.datas.utils.times import get_now_time


@single_task()
def cron_check_db_sleep():
    with scheduler.app.app_context():
        db.session.execute("select 1 limit 1").first()


'''
定时操作
'''


@single_task()
def cron_do(cron_id):
    with scheduler.app.app_context():

        try:
            nows = get_now_time()

            cif = CronInfos.query.get(cron_id)

            if not cif:
                jl = JobLog(cron_info_id=cron_id, content="定时任务不存在", create_time=nows, take_time=0)
                db.session.add(jl)
                db.session.commit()
            else:
                req_url = cif.req_url
                if not req_url:
                    jl = JobLog(cron_info_id=cron_id, content="请求链接不存在", create_time=nows, take_time=0)
                    db.session.add(jl)
                    db.session.commit()
                else:
                    if req_url.find('http') == -1:
                        jl = JobLog(cron_info_id=cron_id, content="请求链接有误，请检查一下", create_time=nows, take_time=0)
                        db.session.add(jl)
                        db.session.commit()
                    else:
                        try:

                            xiaoniu_cron_log_id = str(uuid.uuid1())

                            t = time.time()

                            req = requests.get(
                                req_url, params={'xiaoniu_cron_log_id': xiaoniu_cron_log_id},
                                timeout=2 * 60, headers={'user-agent': 'xiaoniu_cron'})

                            ret = req.text

                            try:
                                ret = req.json()
                            except:
                                pass

                            if type(ret) == dict:
                                ret = json.dumps(ret, ensure_ascii=False)

                            error_keyword = configs('error_keyword')

                            if error_keyword:
                                error_keyword = error_keyword.replace('，', ',').split(',')
                                for item in error_keyword:
                                    if item.strip().lower() in ret.lower():
                                        wechat_info_err('定时任务【%s】发生错误' % cif.task_name, '返回信息:%s' % ret)
                                        break

                            jl = JobLog(cron_info_id=cron_id, content=ret, create_time=nows,
                                        take_time=time.time() - t, log_id=xiaoniu_cron_log_id)
                            db.session.add(jl)
                            db.session.commit()
                        except Exception as e:
                            jl = JobLog(cron_info_id=cron_id, content="发生严重错误:%s" %
                                        str(e), create_time=nows, take_time=time.time() - t)
                            db.session.add(jl)
                            db.session.commit()

                            wechat_info_err('定时任务【%s】发生严重错误' % cif.task_name, '返回信息:%s' % str(e))

        except Exception as e:
            trace_info = traceback.format_exc()
            current_app.logger.error("==============")
            current_app.logger.error(str(e))
            current_app.logger.error(trace_info)
            current_app.logger.error("==============")
            wechat_info_err('定时任务发生严重错误', '返回信息:%s' % str(e))

    return "ok"


@single_task()
def cron_check():
    with scheduler.app.app_context():
        try:
            def dbs():
                url = current_app.config.get('CRON_DB_URL')
                db = records.Database(url)
                db = db.get_connection()  # 新加
                return db

            job_db = dbs()
            job_arr = []
            jobs = job_db.query("select id from apscheduler_jobs").all()
            if jobs:
                for item in jobs:
                    job_arr.append(item.id)

            cifs = CronInfos.query.all()

            if cifs:
                for item in cifs:
                    if "cron_%s" % item.id not in job_arr:
                        item.status = -1
                        db.session.add(item)
                        db.session.commit()
        except Exception as e:
            trace_info = traceback.format_exc()
            current_app.logger.error("==============")
            current_app.logger.error(str(e))
            current_app.logger.error(trace_info)
            current_app.logger.error("==============")
            wechat_info_err('定时任务发生严重错误', '返回信息:%s' % str(e))
    return "ok"


'''
保留一千条数据
'''


@single_task()
def cron_del_job_log():
    with scheduler.app.app_context():
        try:
            job_log_counts = configs('job_log_counts') or 0
            if int(job_log_counts) != 0:
                crons = CronInfos.query.all()
                for item in crons:
                    counts = JobLog.query.filter(JobLog.cron_info_id == item.id).count()
                    if counts > int(job_log_counts):
                        sql = "delete from job_log where cron_info_id = '%s' limit %s" % (
                            item.id, (counts - int(job_log_counts)))
                        db.session.execute(sql)
                        db.session.commit()
        except Exception as e:
            trace_info = traceback.format_exc()
            current_app.logger.error("==============")
            current_app.logger.error(str(e))
            current_app.logger.error(trace_info)
            current_app.logger.error("==============")
    return "ok"




from flask import request
from notejob.center.app.core import db, scheduler
from notejob.center.app.crons import cron_do
from notejob.center.app.decorated import api_deal_return, api_err_return
from notejob.center.configs import configs
from notejob.center.datas.model import CronInfos, JobLog, JobLogItems

from .core import api

'''
添加（更新）定时
access_token
task_name 任务名称唯一
task_keyword 备注
run_date 执行时间
day
day_of_week
hour
minute
second
req_url
'''


@api.route('/cron/add', methods=['GET', 'POST'])
@api.route('/cron', methods=['GET', 'POST'])
@api_deal_return
def crons():

    api_access_token = configs('api_access_token')

    datas = request.values.to_dict()

    task_name = datas.get('task_name')

    task_keyword = datas.get('task_keyword') or ''

    access_token = datas.get('access_token')

    if api_access_token:

        if not access_token:
            return api_err_return(msg='access_token不能为空')

        if api_access_token != access_token:
            return api_err_return(msg='access_token错误')

    if not task_name:
        return api_err_return(msg='任务名称不能为空')

    cif = CronInfos.query.filter(CronInfos.task_name == task_name).first()

    run_date = datas.get('run_date') or ''

    day = datas.get('day') or ''

    if day:
        if day.isdigit() and int(day) not in range(1, 32):
            return api_err_return(msg='日（号）不在范围(0~31)内，请检查！')
        else:
            pass

    day_of_week = datas.get('day_of_week') or ''

    if day_of_week:
        if day_of_week.isdigit():
            if int(day_of_week) not in range(0, 7):
                return api_err_return(msg='星期 不在范围(0~6)内，请检查！')
        else:
            if day_of_week not in ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']:
                return api_err_return(msg='星期 不在范围(mon,tue,wed,thu,fri,sat,sun)内，请检查！')

    hour = datas.get('hour') or ''

    if hour and hour.isdigit():
        if int(hour) not in range(0, 24):
            return api_err_return(msg='小时 不在范围(0~23)内，请检查！')

    minute = datas.get('minute') or ''
    if minute and minute.isdigit():
        if int(minute) not in range(0, 60):
            return api_err_return(msg='分钟 不在范围(0~59)内，请检查！')

    second = datas.get('second') or ''

    if second and second.isdigit():
        if int(second) not in range(0, 60):
            return api_err_return(msg='秒 不在范围(0~59)内，请检查！')

    '''
    判断一下 run_date 必须有个不能为空
    '''
    if not run_date:
        if not day_of_week and not day and not hour and not minute and not second:
            return api_err_return(msg='信息请完整填写！')

    req_url = datas.get('req_url')

    if not req_url:
        return api_err_return(msg='回调URL(req_url)必填！')

    if 'http://' not in req_url and 'https://' not in req_url:
        return api_err_return(msg='回调URL格式有错')

    if not cif:
        cif = CronInfos(task_name=task_name, task_keyword=task_keyword, run_date=run_date, day_of_week=day_of_week,
                        day=day, hour=hour, minute=minute, second=second, req_url=req_url, status=1)
    else:
        cif.task_name = task_name
        cif.task_keyword = task_keyword
        cif.run_date = run_date
        cif.day_of_week = day_of_week
        cif.day = day
        cif.hour = hour
        cif.minute = minute
        cif.second = second
        cif.req_url = req_url
        cif.status = 1

    db.session.add(cif)
    db.session.commit()

    cron_id = cif.id

    cron_datas = {}

    if run_date:
        cron_datas['trigger'] = 'date'
        cron_datas['run_date'] = run_date
    else:
        # 定时的
        cron_datas['trigger'] = 'cron'
        if day_of_week:
            cron_datas['day_of_week'] = day_of_week
        if hour:
            cron_datas['hour'] = hour
        if minute:
            cron_datas['minute'] = minute
        if day:
            cron_datas['day'] = day
        if second and second != '*':
            cron_datas['second'] = second

    scheduler.add_job("cron_%s" % cron_id, func=cron_do, args=[cron_id], replace_existing=True, **cron_datas)

    return 'ok'


'''
更新状态
task_name 任务名称
access_token 
status
'''


@api.route('/cron/status', methods=['GET', 'POST'])
@api_deal_return
def cron_status():
    datas = request.values.to_dict()

    api_access_token = configs('api_access_token')

    task_name = datas.get('task_name')

    access_token = datas.get('access_token')

    status = datas.get('status')

    if status:
        try:
            if int(status) not in [0, 1]:
                return api_err_return(msg='status只能0或者1')
        except:
            return api_err_return(msg='status只能0或者1')

    if api_access_token:

        if not access_token:
            return api_err_return(msg='access_token不能为空')

        if api_access_token != access_token:
            return api_err_return(msg='access_token错误')

    if not task_name:
        return api_err_return(msg='任务名称不能为空')

    ci = CronInfos.query.filter(CronInfos.task_name == task_name).first()
    if not ci:
        return api_err_return(msg='任务不存在')

    if ci.status == -1:
        return api_err_return(msg='任务已结束，不能再操作，只能重新更新')

    if not status:
        # 0停止1运行中
        if ci.status == 0:
            # 开启
            ci.status = 1
            scheduler.resume_job('cron_%s' % ci.id)
        else:
            ci.status = 0
            # 关闭
            scheduler.pause_job('cron_%s' % ci.id)
    else:
        if int(status) == 0 and ci.status != 0:
            ci.status = 0
            # 关闭
            scheduler.pause_job('cron_%s' % ci.id)

        if int(status) == 1 and ci.status != 1:
            ci.status = 1
            scheduler.resume_job('cron_%s' % ci.id)

    db.session.add(ci)
    db.session.commit()

    return 'ok'


'''
上传执行记录
xiaoniu_cron_log_id
content
'''


@api.route('/cron/add_log', methods=['GET', 'POST'])
@api_deal_return
def cron_add_log():
    datas = request.values.to_dict()

    api_access_token = configs('api_access_token')

    access_token = datas.get('access_token')

    xiaoniu_cron_log_id = datas.get('xiaoniu_cron_log_id')

    if api_access_token:

        if not access_token:
            return api_err_return(msg='access_token不能为空')

        if api_access_token != access_token:
            return api_err_return(msg='access_token错误')

    if not xiaoniu_cron_log_id:
        return api_err_return(msg='xiaoniu_cron_log_id 必传哦！')

    content = datas.get('content')
    if not content:
        return api_err_return(msg='日志内容不能为空')

    jl = JobLog.query.filter(JobLog.log_id == xiaoniu_cron_log_id).first()
    if not jl:
        return api_err_return(msg='xiaoniu_cron_log_id 不存在')
    jli = JobLogItems(log_id=xiaoniu_cron_log_id, content=content)
    db.session.add(jli)
    db.session.commit()
    return 'ok'



# -*- coding:utf-8 -*-
import traceback

from flask import current_app, redirect, render_template, request, session
from notejob.center.app.common.functions import web_api_return, wechat_info_err
from notejob.center.app.core import db, scheduler
from notejob.center.app.crons import cron_do
from notejob.center.app.decorated import login_required
from notejob.center.datas.model import CronInfos, JobLog, JobLogItems
from notejob.center.datas.utils.times import get_now_time

from.core import blue_print


@blue_print.route('/cron_list', methods=['GET', 'POST'])
@blue_print.route('/', methods=['GET', 'POST'])
@login_required
def cron_list():
    keyword = request.args.to_dict()
    page = int(request.args.get('page') or 1)
    task_name = keyword.get('task_name')
    filter_arr = []
    if task_name:
        filter_arr.append(CronInfos.task_name.like('%{}%'.format(task_name)))

    page_data = CronInfos.query.filter(*filter_arr).order_by(db.desc(CronInfos.task_name)).paginate(page=page,
                                                                                                    per_page=20)
    if 'page' in keyword:
        del keyword['page']
    return render_template("cron_list.html", page_data=page_data, keyword=keyword)


@blue_print.route('/api_doc', methods=['GET', 'POST'])
@login_required
def api_doc():
    return render_template("api_doc.html")


@blue_print.route('/job_log_list', methods=['GET', 'POST'])
@login_required
def job_log_list():
    keywords = request.args.to_dict()

    page = int(request.args.get('page') or 1)
    id = request.args.get('id')

    page_data = JobLog.query.filter(
        JobLog.cron_info_id == id).order_by(
        db.desc(JobLog.id)).paginate(
        page=page, per_page=20)
    if 'page' in keywords:
        del keywords['page']

    return render_template("job_log_list.html", page_data=page_data, keywords=keywords)


@blue_print.route('/job_log_item_list', methods=['GET', 'POST'])
@login_required
def job_log_item_list():
    log_id = request.args.get('log_id')
    page_data = JobLogItems.query.filter(JobLogItems.log_id == log_id).all()

    return render_template("job_log_item_list.html", page_data=page_data)


@blue_print.route('/job_log_all_list', methods=['GET', 'POST'])
@login_required
def job_log_all_list():
    keywords = request.args.to_dict()

    page = int(request.args.get('page') or 1)

    filter_arr = []
    task_name = keywords.get('task_name')
    if task_name:
        filter_arr.append(CronInfos.task_name.like('{}%'.format(task_name)))
    beg_time = keywords.get('beg_time')
    end_time = keywords.get('end_time')
    if beg_time and end_time:
        filter_arr.append(JobLog.create_time.between(beg_time, end_time))

    page_data = JobLog.query.\
        join(CronInfos, CronInfos.id == JobLog.cron_info_id).\
        filter(*filter_arr).\
        order_by(db.desc(JobLog.id)).\
        add_entity(CronInfos).\
        paginate(page=page, per_page=20)

    if 'page' in keywords:
        del keywords['page']

    return render_template("job_log_all_list.html", page_data=page_data, keywords=keywords)


@blue_print.route('/job_log_delete', methods=['GET', 'POST'])
@login_required
def job_log_delete():
    datas = request.values.to_dict()
    job_log_id = datas.get('job_log_id')
    job_logs = JobLog.query.get(job_log_id)
    if not job_logs:
        return web_api_return(code=1, msg='信息不存在')
    db.session.delete(job_logs)
    db.session.commit()

    return web_api_return(code=0, msg='删除成功')


@blue_print.route('/job_batch_delete', methods=['GET', 'POST'])
@login_required
def job_batch_delete():
    ids = request.form.getlist('id')
    JobLog.query.filter(JobLog.id.in_(ids)).delete(synchronize_session=False)
    db.session.commit()
    return web_api_return(code=0, msg='操作成功', url='/job_log_all_list')


@blue_print.route('/cron_add', methods=['GET', 'POST'])
@login_required
def cron_add():
    if request.method == 'POST':
        try:
            datas = request.values.to_dict()
            task_name = datas.get('task_name')
            task_keyword = datas.get('task_keyword')

            if not task_name:
                return web_api_return(code=1, msg='任务名称不能为空')

            _cif = CronInfos.query.filter(CronInfos.task_name == task_name).first()
            if _cif:
                return web_api_return(code=1, msg='任务名称已存在')

            run_date = datas.get('run_date')
            if run_date:
                if run_date < get_now_time('%Y-%m-%d %H:%M'):
                    return web_api_return(code=1, msg='设置的时间已过期，请重新设置')
            day = datas.get('day')

            if day:
                if day.isdigit() and int(day) not in range(1, 32):
                    return web_api_return(code=1, msg='日（号）不在范围内，请检查！')
                else:
                    pass

            day_of_week = datas.get('day_of_week')

            if day_of_week:
                if day_of_week.isdigit():
                    if int(day_of_week) not in range(0, 7):
                        return web_api_return(code=1, msg='星期 不在范围内，请检查！')
                else:
                    if day_of_week not in ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']:
                        return web_api_return(code=1, msg='星期 不在范围内，请检查！')

            hour = datas.get('hour')

            if hour and hour.isdigit():
                if int(hour) not in range(0, 24):
                    return web_api_return(code=1, msg='小时 不在范围内，请检查！')

            minute = datas.get('minute')
            if minute and minute.isdigit():
                if int(minute) not in range(0, 60):
                    return web_api_return(code=1, msg='分钟 不在范围内，请检查！')

            second = datas.get('second')

            if second and second.isdigit():
                if int(second) not in range(0, 60):
                    return web_api_return(code=1, msg='秒 不在范围内，请检查！')

            ds_ms = datas.get('ds_ms')
            if ds_ms == '1':
                if not run_date:
                    return web_api_return(code=1, msg='时间没设置呢！')
            else:
                if not day_of_week and not day and not hour and not minute and not second:
                    return web_api_return(code=1, msg='请完整填写！')

            req_url = datas.get('req_url')

            if not req_url:
                return web_api_return(code=1, msg='回调URL必填！')

            if 'http://' not in req_url and 'https://' not in req_url:
                return web_api_return(code=1, msg='URL格式有误！')
            cif = CronInfos(task_name=task_name, task_keyword=task_keyword, run_date=run_date, day_of_week=day_of_week,
                            day=day, hour=hour, minute=minute, second=second, req_url=req_url, status=1)

            db.session.add(cif)
            db.session.commit()

            cron_id = cif.id

            cron_datas = {}

            if run_date:
                cron_datas['trigger'] = 'date'
                cron_datas['run_date'] = run_date
            else:
                # 定时的
                cron_datas['trigger'] = 'cron'
                if day_of_week:
                    cron_datas['day_of_week'] = day_of_week
                if hour:
                    cron_datas['hour'] = hour
                if minute:
                    cron_datas['minute'] = minute
                if day:
                    cron_datas['day'] = day
                if second and second != '*':
                    cron_datas['second'] = second

            scheduler.add_job("cron_%s" % cron_id, func=cron_do, args=[cron_id], replace_existing=True, **cron_datas)
            return web_api_return(code=0, msg='添加成功', url='/cron_list')

        except Exception as e:
            trace_info = traceback.format_exc()
            wechat_info_err(str(e), trace_info)
            return web_api_return(code=1, msg=str(e), url='/cron_list')

    return render_template("cron_add.html")


@blue_print.route('/cron_edit', methods=['GET', 'POST'])
@login_required
def cron_edit():
    id = request.values.get('id')
    cif = CronInfos.query.get(id)
    if request.method == 'POST':

        datas = request.values.to_dict()
        ds_ms = datas.get('ds_ms')
        task_name = datas.get('task_name')

        if not task_name:
            return web_api_return(code=1, msg='任务名称不能为空')

        task_keyword = datas.get('task_keyword') or ''

        _cif = CronInfos.query.filter(CronInfos.task_name == task_name, CronInfos.id != datas.get('id')).first()
        if _cif:
            return web_api_return(code=1, msg='任务名称已存在已存在')

        run_date = datas.get('run_date')

        if ds_ms == '2':
            run_date = ''

        if run_date:
            if run_date < get_now_time('%Y-%m-%d %H:%M'):
                return web_api_return(code=1, msg='设置的时间已过期，请重新设置')

        day = datas.get('day')

        if day:
            if day.isdigit() and int(day) not in range(1, 32):
                return web_api_return(code=1, msg='日（号）不在范围内，请检查！')
            else:
                pass

        day_of_week = datas.get('day_of_week')

        if day_of_week:
            if day_of_week.isdigit():
                if int(day_of_week) not in range(0, 7):
                    return web_api_return(code=1, msg='星期 不在范围内，请检查！')
            else:
                if day_of_week not in ['mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']:
                    return web_api_return(code=1, msg='星期 不在范围内，请检查！')

        hour = datas.get('hour')

        if hour and hour.isdigit():
            if int(hour) not in range(0, 24):
                return web_api_return(code=1, msg='小时 不在范围内，请检查！')

        minute = datas.get('minute')
        if minute and minute.isdigit():
            if int(minute) not in range(0, 60):
                return web_api_return(code=1, msg='分钟 不在范围内，请检查！')

        second = datas.get('second')

        if second and second.isdigit():
            if int(second) not in range(0, 60):
                return web_api_return(code=1, msg='秒 不在范围内，请检查！')

        ds_ms = datas.get('ds_ms')
        if ds_ms == '1':
            if not run_date:
                return web_api_return(code=1, msg='时间没设置呢！')
        else:
            if not day_of_week and not day and not hour and not minute and not second:
                return web_api_return(code=1, msg='请完整填写！')

        req_url = datas.get('req_url')

        if not req_url:
            return web_api_return(code=1, msg='回调URL必填！')

        if 'http://' not in req_url and 'https://' not in req_url:
            return web_api_return(code=1, msg='URL格式有误！')

        cif.task_name = task_name
        cif.task_keyword = task_keyword
        cif.run_date = run_date
        cif.day_of_week = day_of_week
        cif.day = day
        cif.hour = hour
        cif.minute = minute
        cif.second = second
        cif.req_url = req_url
        cif.status = 1
        db.session.add(cif)

        db.session.commit()

        cron_id = cif.id

        cron_datas = {}
        if run_date:
            cron_datas['trigger'] = 'date'
            cron_datas['run_date'] = run_date
        else:
            # 定时的
            cron_datas['trigger'] = 'cron'
            if day_of_week:
                cron_datas['day_of_week'] = day_of_week
            if hour:
                cron_datas['hour'] = hour
            if minute:
                cron_datas['minute'] = minute
            if day:
                cron_datas['day'] = day
            if second and second != '*':
                cron_datas['second'] = second

        scheduler.add_job("cron_%s" % cron_id, func=cron_do, args=[cron_id], replace_existing=True, **cron_datas)

        return web_api_return(code=0, msg='修改成功！', url='/cron_list')

    return render_template("cron_edit.html", cif=cif)


@blue_print.route('/update_status', methods=['GET', 'POST'])
@login_required
def update_status():
    id = request.args.get('id')
    cif = CronInfos.query.get(id)
    if not cif:
        return web_api_return(code=1, msg='项目不存在', url='/cron_list')
    status = cif.status
    _status = 0
    if status == 0:
        _status = 1
        scheduler.resume_job('cron_%s' % cif.id)
    else:
        scheduler.pause_job('cron_%s' % cif.id)
    cif.status = _status
    db.session.add(cif)
    db.session.commit()
    return web_api_return(code=0, msg='操作成功')


@blue_print.route('/cron_del', methods=['GET', 'POST'])
@login_required
def cron_del():
    id = request.args.get('id')
    cif = CronInfos.query.get(id)
    if not cif:
        return web_api_return(code=1, msg='项目不存在', url='/cron_list')
    cron_id = cif.id

    db.session.delete(cif)

    try:
        scheduler.remove_job('cron_%s' % cron_id)
    except:
        pass

    db.session.execute("delete from job_log where cron_info_id='%s'" % cron_id)

    db.session.commit()
    return web_api_return(code=0, msg='操作成功', url='/cron_list')


@blue_print.route('/cron_batch_del', methods=['GET', 'POST'])
@login_required
def cron_batch_del():
    ids = request.form.getlist('id')
    CronInfos.query.filter(CronInfos.id.in_(ids)).delete(synchronize_session=False)
    JobLog.query.filter(JobLog.cron_info_id.in_(ids)).delete(synchronize_session=False)
    db.session.commit()

    try:
        for cron_id in ids:
            scheduler.remove_job('cron_%s' % cron_id)
    except:
        pass

    return web_api_return(code=0, msg='操作成功', url='/cron_list')


@blue_print.route('/check_pass', methods=['GET', 'POST'])
def check_pass():
    today = get_now_time()
    msg = request.values.get('msg', '')

    if request.method == 'POST':
        try:
            password = request.values.get('password')
            if not password:
                return redirect("/check_pass?msg=密码不能为空")
            login_pwd = current_app.config.get('LOGIN_PWD')
            if not login_pwd:
                return redirect("/check_pass?msg=请联系管理员")
            if login_pwd != password:
                return redirect("/check_pass?msg=密码有误")
            session['is_login'] = True
            return redirect('/cron_list')
        except:
            return redirect("/check_pass?msg=系统有误,请重新试试")
    return render_template("check_pass.html", msg=msg, today=today)


@blue_print.route('/logout')
def logout():
    session.clear()
    return redirect("/check_pass")



import os

from apscheduler.jobstores.sqlalchemy import SQLAlchemyJobStore

from .configs import configs

basedir = os.path.abspath(os.path.dirname(__file__))

redis_host = configs('redis_host')


class Config:
    JSON_AS_ASCII = False
    JSONIFY_PRETTYPRINT_REGULAR = False
    SECRET_KEY = os.environ.get('SECRET_KEY') or 'hard to guess string'
    SQLALCHEMY_COMMIT_ON_TEARDOWN = False
    SQLALCHEMY_TRACK_MODIFICATIONS = False

    SCHEDULER_API_ENABLED = False

    CRON_DB_URL = configs('cron_db_url')

    BASEDIR = basedir

    SCHEDULER_JOBSTORES = {
        'default': SQLAlchemyJobStore(url=configs('cron_db_url'))
    }
    SCHEDULER_EXECUTORS = {
        'default': {
            'type': 'threadpool',
            'max_workers': 30
        }
    }
    # 'misfire_grace_time':30
    SCHEDULER_JOB_DEFAULTS = {
        'coalesce': False,
        'max_instances': 20,
        'misfire_grace_time': 50
    }

    JOBS = [
        {
            'id': 'cron_check',
            'func': 'notejob.center.app.crons:cron_check',
            'args': None,
            'replace_existing': True,
            'trigger': 'cron',
            'day_of_week': "*",
            'day': '*',
            'hour': '*',
            'minute': '*/30'
        },
        {
            'id': 'cron_del_job_log',
            'func': 'notejob.center.app.crons:cron_del_job_log',
            'args': None,
            'replace_existing': True,
            'trigger': 'cron',
            'day_of_week': "*",
            'day': '*',
            'hour': '*/8'
        },
        {
            'id': 'cron_check_db_sleep',
            'func': 'notejob.center.app.crons:cron_check_db_sleep',
            'args': None,
            'replace_existing': True,
            'trigger': 'cron',
            'day_of_week': "*",
            'day': '*',
            'hour': '*',
            'minute': '*/10',
        }
    ]

    LOGIN_PWD = configs('login_pwd')

    @staticmethod
    def init_app(app):
        logs_path = os.path.join(basedir, 'datas/logs')
        if not os.path.exists(logs_path):
            os.mkdir(logs_path)


class DevelopmentConfig(Config):
    DEBUG = True

    SQLALCHEMY_DATABASE_URI = configs('cron_job_log_db_url')


class TestingConfig(Config):
    TESTING = True
    SQLALCHEMY_DATABASE_URI = os.environ.get('TEST_DATABASE_URL') or \
        'sqlite:///' + os.path.join(basedir, 'data-test.sqlite')


class ProductionConfig(Config):
    DEBUG = False
    SQLALCHEMY_DATABASE_URI = configs('cron_job_log_db_url')


config = {
    'development': DevelopmentConfig,
    'testing': TestingConfig,
    'production': ProductionConfig,
    'default': DevelopmentConfig
}

config_dict = {
    'development': DevelopmentConfig,
    'testing': TestingConfig,
    'production': ProductionConfig,
    'default': DevelopmentConfig
}



from.core import blue_print


@blue_print.app_errorhandler(404)
def page_not_found(e):
    return 'page not found'


@blue_print.app_errorhandler(500)
def internal_server_error(e):
    return 'system err'




from notejob.center.app.core import db


class JobLogItems(db.Model):
    __tablename__ = 'job_log_items'
    id = db.Column(db.Integer, primary_key=True)
    log_id = db.Column(db.String(65), index=True, nullable=False)
    content = db.Column(db.TEXT, nullable=False, default='')
